@@ -5,8 +5,8 @@
 # Gibbs Sampler Function
   .gibbs_IFA     <- function(Q, data, iters, N, P, sigma.mu, mu, prop, truncated, uni.type,
                              uni.prior, psi.alpha, psi.beta, burnin, thinning, verbose,
-                             sw, epsilon, mu.zero, nu1, nu2, adapt, start.AGS, stop.AGS,
-                             b0, b1, alpha.d1, alpha.d2, beta.d1, beta.d2, scaling, ...) {
+                             sw, epsilon, mu.zero, nu1, nu2, adapt, active.crit, start.AGS, stop.AGS,
+                             b0, b1, alpha.d1, alpha.d2, beta.d1, beta.d2, scaling, col.mean, ...) {
 
   # Define & initialise variables
     start.time   <- proc.time()
@@ -66,7 +66,7 @@
     tau          <- cumprod(delta)
     lmat         <- matrix(vapply(Pseq, function(j) .sim_load_ps(Q=Q, phi=phi[j,], tau=tau), numeric(Q)), nrow=P, byrow=TRUE)
     psi.inv      <- .sim_psi_ip(P=P, psi.alpha=psi.alpha, psi.beta=psi.beta)
-    psi.inv[]    <- 1/switch(EXPR=uni.type, constrained=colVars(data), max(colVars(data)))
+    psi.inv[]    <- 1/switch(EXPR=uni.type, constrained=colVars(data, center=col.mean, refine=FALSE, useNames=FALSE), max(colVars(data, center=col.mean, refine=FALSE, useNames=FALSE)))
     max.p        <- (psi.alpha  - 1)/psi.beta
     inf.ind      <- psi.inv > max(max.p)
     psi.inv[inf.ind]       <- switch(EXPR=uni.type, constrained=max.p, rep(max.p, P))[inf.ind]
@@ -82,8 +82,19 @@
     # Adaptation
       if(adapt   && all(iter >= start.AGS, iter < stop.AGS))      {
         if(stats::runif(1) < ifelse(iter < AGS.burn, 0.5, exp(-b0 - b1 * (iter - start.AGS)))) {
-          colvec <- if(Q0)  (colSums2(abs(lmat) < epsilon) / P)  >= prop else stats::runif(1) <= prop
-          numred <- sum(colvec)
+          switch(EXPR=active.crit, SC={
+            if(Q0)   {
+              SC <- .SC_crit(data, eta, lmat, prop)
+              nonred      <- SC$nonred
+              numred      <- SC$numred
+            } else  {
+              colvec      <- stats::runif(1)    <= prop
+              numred      <- sum(colvec)
+            }
+          },        {
+            colvec        <- if(Q0) (colSums2(abs(lmat) < epsilon, useNames=FALSE) / P) >= prop else stats::runif(1) <= prop
+            numred        <- sum(colvec)
+          })
           if(numred == 0)  {
             Q    <- Q + 1L
             Q.big   <- Q   > Q.star
@@ -95,8 +106,8 @@
               tau   <- cumprod(delta)
               lmat  <- cbind(lmat, stats::rnorm(n=P, mean=0, sd=1/sqrt(phi[,Q] * tau[Q])))
             }
-          } else if(Q0)         {
-            nonred  <- colvec  == 0
+          } else if(Q0)    {
+            nonred  <- switch(EXPR=active.crit, BD=colvec == 0, nonred)
             Q       <- max(0L, Q - numred)
             phi     <- phi[,nonred, drop=FALSE]
             delta   <- delta[nonred]
@@ -125,14 +136,14 @@
 
     # Means
       if(update.mu) {
-        mu[]     <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta), lmat=lmat, mu.prior=mu.prior)
+        mu[]     <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta, useNames=FALSE), lmat=lmat, mu.prior=mu.prior)
       }
 
     # Shrinkage
       if(Q0) {
         load.2   <- lmat^2
         phi      <- .sim_phi(Q=Q, P=P, nu1.5=nu1.5, nu2=nu2, tau=tau, load.2=load.2)
-        sum.term <- colSums2(phi * load.2)
+        sum.term <- colSums2(phi * load.2, useNames=FALSE)
         for(k in seq_len(Q)) {
           delta[k]  <- if(k > 1) .sim_deltak(alpha.d2=alpha.d2, beta.d2=beta.d2, delta.k=delta[k], Q=Q, P.5=P.5, k=k,
                        tau.kq=tau[k:Q], sum.term.kq=sum.term[k:Q]) else .sim_delta1(Q=Q, P.5=P.5, tau=tau, sum.term=sum.term,
@@ -141,7 +152,7 @@
         }
       }
 
-      if(Q.big && !Q.large && iter > burnin) {       cat("\n"); warning(paste0("\nQ has exceeded initial number of loadings columns since burnin: consider increasing range.Q from ", Q.star, "\n"), call.=FALSE)
+      if(Q.big && !Q.large && iter > burnin) {       cat("\n"); warning(paste0("\nQ has exceeded initial number of loadings columns since burnin: consider increasing range.Q from ", Q.star, "\n"), call.=FALSE, immediate.=TRUE)
         Q.large  <- TRUE
       }
       if(storage) {
